#!/usr/bin/env python
import sys, os, math, string, re, gzip, urllib, shutil, Bio
import cStringIO
from numpy import *
#from Bio.Blast.Applications import NcbipsiblastCommandline
from Bio.Blast.Applications import *
import get_PDB_seq, alignment


#-------# Input:
dssp_cmd = 'dsspcmbi.exe' # DSSP executable, must be in local directory
blast_cmd= r"C:\\NCBI\\blast-2.2.24+\\bin\\psiblast.exe" # Blast executable
blast_database = "C:\\blast\\db2\\nr" # Blast database

#-------# Outputs:
g = open( "alignment_sizes.txt", 'a' )
b = open( "betas.txt", 'a')
align = open("dissimilar_alignments.txt", 'a')



residue_hydrophobicity = {'A': .31, 'R': -1.01, 'N': -0.60, 'D': -0.77, \
		   'C': 1.54, 'Q': -0.22, 'E': -.64, 'G': 0.00, \
		   'H': 0.13, 'I': 1.80, 'L': 1.70, 'K': -0.99, \
		   'M': 1.23, 'F': 1.79, 'P': 0.75, 'S': -0.04, \
		   'T': 0.26, 'W': 2.25, 'Y': 0.96, 'V': 1.22}


#-------# RETURNS A LIST OF PDB NAMES
def get_sequence_list( protein_list ):
    pdb_list = []
    f = open( protein_list, 'rb')
    for line in f:
        pdb_list.append(line[:-1])
    f.close()
    return pdb_list



#-------# THIS RETURNS THE ACTUAL SEQUENCE TO BLAST, (SEQ, RSA)
def get_sequence( pdb_entry ):
    pdb_id = pdb_entry[:4]
    chain_id = pdb_entry[-1]
    current_sequences = get_PDB_seq.getStruInfo( pdb_id, chain_id, DSSP = dssp_cmd )
    return current_sequences



#-------# Returns protein sequence and bins
def bin_sites( pdb_entry ):

    RESseq, RSAseq = get_sequence( pdb_entry )
    endpoints = range(11)
    RSAbins = [[] for i in range(10)]
    for i in range(len(RSAseq)):
        for j in range(10):
            if RSAseq[i]*(10) < endpoints[j+1]:
                RSAbins[j].append(i)
                break
            elif RSAseq[i]*(10-1) > endpoints[-1]:
                RSAbins[-1].append(i)
                break
            continue
    
    return RESseq, RSAbins


#------# Blasts whatever's in temp_fasta and returns dissimilar alignment matrix.
def blast_em( pdb, current_seq_len ):
    matrix = []
    dissimilar_matrix = []

    blast_output = pdb + "_blast.xml"
    if not os.access( blast_output, os.F_OK ):
        cline = NcbipsiblastCommandline(cmd = blast_cmd, db = blast_database,
                                        query="temp_fasta.txt",
                                        out = pdb + "_blast.xml",
                                        outfmt=5, num_iterations=2)
        cline()
    dissimilar_matrix = alignment.formDissimilarAlignmentMatrix( pdb, current_seq_len )

    os.unlink("temp_fasta.txt")
    if len(dissimilar_matrix) > 49:
        align.write(str(pdb) + '\r\n')
        for i in range(len(dissimilar_matrix)):
            for j in range(len(dissimilar_matrix[i])):
                align.write(str(dissimilar_matrix[i][j]))
            align.write('\r\n')
        align.write('\r\n')
        align.close()

    return dissimilar_matrix



def compute_distributions( dissim_alignment, RSAbins, bin_number):
    Alignment = dissim_alignment

    sitesInBin = []
    sitesInBin.extend(RSAbins[bin_number])
    n = len(Alignment)
    m = len(sitesInBin)
    distributionMatrix = [[] for i in range(m)]
    for i in range(m):
        for j in range(n):
            if not Alignment[j][sitesInBin[i]] in residue_hydrophobicity.keys():
                break
            else:
            #if not Alignment[j][sitesInBin[i]] == "-":
                #if not Alignment[j][sitesInBin[i]] == "X":
                distributionMatrix[i].append(str(residue_hydrophobicity[Alignment[j][sitesInBin[i]]]))

    OrderedHPs = [str(-1.01), str(-0.99), str(-0.77), str(-0.64), str(-0.6), str(-0.22), str(-0.04), str(0.0), str(0.13), str(0.26), str(0.31), str(0.75), str(0.96), str(1.22), str(1.23), str(1.54), str(1.7), str(1.79), str(1.8), str(2.25)]

    NewBins = zeros([m,20], int)
    for i in range(m):
        for j in range(len(distributionMatrix[i])):
            k = OrderedHPs.index(distributionMatrix[i][j])
            NewBins[i][k] = NewBins[i][k] + int(1)
    NormalizedBins = zeros([m,20], float)
    for i in range(m):
        rowsum = 0.0
        for k in range(20):
            rowsum = rowsum + NewBins[i][k]
        for j in range(20):
            NormalizedBins[i][j] = NewBins[i][j]/rowsum
    #print NormalizedBins
    for i in range(m):
            NewBins[i].sort()
    for i in range(m):
            NormalizedBins[i].sort()


    binAverage = zeros(20, float)
    for j in range(20):
        for i in range(len(NewBins)):
            binAverage[j] = int(binAverage[j]) + int(NewBins[i][j])
        lastguy = binAverage[-1]
        for j in range(20):
            if lastguy == 0:
                break
            else:
                binAverage[j] = binAverage[j]/lastguy
    return binAverage
    
            

    


def do_math(matrix):
    Blist = []*10
    B_weightedlist = []*10


    matrix2 = [[] for i in range(10)]
    for i in range(10):
        for j in range(20):
            matrix2[i].append(matrix[i][-(j+1)])
    for i in range(10):
        print matrix2[i], '\n'

    for i in range(10):
        Sxx = SxlnY = SlnY = SBx2 = SlnY2 = errors = exp_errors = 0.0
        Sxxy = Sxylny = Slny = Sy = 0.0
        Y = matrix2[i][:]
        for j in range(20):
            if not Y[-1] > 0.0:
                del Y[-1]
            else:
                break

        n = len(Y)
        lnY = [0.0]*n
        for k in range(n):
            try:
                lnY[k] = math.log(Y[k])
            except ValueError:
                break
        X = range(n)
        if len(X) != len(Y):
            raise ValueError, 'unequal length'
        """This was weighted with size of Y:"""
        
        for j in range(n):
            Sxxy = Sxxy + X[j]*X[j]*Y[j]
            Sxylny = Sxylny + X[j]*Y[j]*lnY[j]
            Slny = Slny + lnY[j]
        B_weighted = -1*Sxylny/Sxxy

        placeholderB_weighted = str(B_weighted)
        B_weightedlist.append(placeholderB_weighted)


    return B_weightedlist
        
    




def top_level( protein_list ):
    b.write("PDB" + '\t')
    for i in range(10):
        b.write(str(i+1) + ' ')
    b.write('\r\n')

    g.write("PDB" + '\t' + "DissimilarAlignmentSize" + '\t' + "SequenceLength" + '\r\n' )
    
    pdb_list = get_sequence_list(protein_list)
    too_few_orthologs = []
    bad_pdbs = []
    
    for i in range(len(pdb_list)):
    #for i in range(1,2):
        try:
            matrix_to_fit = []
            current_sequences = bin_sites( pdb_list[i] ) #--> (residue sequence, binned sites)
            current_fasta = ''
            for x in current_sequences[0]:
                current_fasta += x
            current_seq_len = len(current_fasta)
            f = open("temp_fasta.txt", 'wb')
            f.write(current_fasta)
            f.close()
            print current_fasta
            dissimilar_matrix = blast_em( pdb_list[i], current_seq_len )
            if len(dissimilar_matrix) < 50:
                too_few_orthologs.append(pdb_list[i])
                continue
            g.write( str(pdb_list[i]) + '\t' + str(len(dissimilar_matrix)) + '\t' + str(current_seq_len) + '\r\n' )
            
            for j in range(10):
                x = compute_distributions( dissimilar_matrix, current_sequences[1], j)
                matrix_to_fit.append(x)

            betas = do_math(matrix_to_fit)
            #print betas
            
            b.write(pdb_list[i] + '\t')
            for i in range(10):
                b.write(betas[i] + ' ')
            b.write('\r\n')

        except:
            bad_pdbs.append(pdb_list[i])
            continue
    print len(too_few_orthologs), "with too few orthologs:"
    print too_few_orthologs
    print "Problem pdbs:", bad_pdbs
    b.close()
    g.close()
    return []







